@@ -183,18 +183,18 @@ krb5_flags_to_string(flags, sep, buffer, buflen)
 
     pflags = 0;
     sepstring = (sep) ? sep : flags_default_sep;
-    krb5int_buf_init_fixed(&buf, buffer, buflen);
+    k5_buf_init_fixed(&buf, buffer, buflen);
     /* Blast through the table matching all we can */
     for (i=0; i<flags_table_nents; i++) {
         if (flags & flags_table[i].fl_flags) {
-            if (krb5int_buf_len(&buf) > 0)
-                krb5int_buf_add(&buf, sepstring);
-            krb5int_buf_add(&buf, _(flags_table[i].fl_output));
+            if (k5_buf_len(&buf) > 0)
+                k5_buf_add(&buf, sepstring);
+            k5_buf_add(&buf, _(flags_table[i].fl_output));
             /* Keep track of what we matched */
             pflags |= flags_table[i].fl_flags;
         }
     }
-    if (krb5int_buf_data(&buf) == NULL)
+    if (k5_buf_data(&buf) == NULL)
         return(ENOMEM);
 
     /* See if there's any leftovers */
