@@ -145,8 +145,7 @@ find_changed_attrs(krb5_db_entry *current, krb5_db_entry *new,
 }
 
 
-/*
- */
+/* Initialize *u with a copy of d.  Return 0 on success, -1 on failure. */
 static int
 data_to_utf8str(utf8str_t *u, krb5_data d)
 {
@@ -249,7 +248,8 @@ set_from_utf8str(krb5_data *d, utf8str_t u)
 static krb5_principal
 conv_princ_2db(krb5_context context, kdbe_princ_t *kdbe_princ)
 {
-    int i;
+    unsigned int i;
+    int j;
     krb5_principal princ;
     kdbe_data_t *components;
 
@@ -276,10 +276,10 @@ conv_princ_2db(krb5_context context, kdbe_princ_t *kdbe_princ)
         princ->data[i].data = NULL;
     princ->length = (krb5_int32)kdbe_princ->k_components.k_components_len;
 
-    for (i = 0; i < princ->length; i++) {
-        princ->data[i].magic = components[i].k_magic;
-        set_from_utf8str(&princ->data[i], components[i].k_data);
-        if (princ->data[i].data == NULL)
+    for (j = 0; j < princ->length; j++) {
+        princ->data[j].magic = components[j].k_magic;
+        set_from_utf8str(&princ->data[j], components[j].k_data);
+        if (princ->data[j].data == NULL)
             goto error;
     }
 
@@ -589,6 +589,7 @@ ulog_conv_2dbentry(krb5_context context, krb5_db_entry **entry,
     krb5_error_code ret;
     unsigned int prev_n_keys = 0;
     krb5_boolean is_add;
+    void *newptr;
 
     *entry = NULL;
 
@@ -687,15 +688,12 @@ ulog_conv_2dbentry(krb5_context context, krb5_db_entry **entry,
             if (is_add)
                 ent->key_data = NULL;
 
-            ent->key_data = (krb5_key_data *)realloc(ent->key_data,
-                                                     (ent->n_key_data *
-                                                      sizeof (krb5_key_data)));
-            /* XXX Memory leak: Old key data in
-               records eliminated by resizing to
-               smaller size.  */
-            if (ent->key_data == NULL)
-                /* XXX Memory leak: old storage.  */
-                return (ENOMEM);
+            /* Allocate one extra key data to avoid allocating zero bytes. */
+            newptr = realloc(ent->key_data, (ent->n_key_data + 1) *
+                             sizeof(krb5_key_data));
+            if (newptr == NULL)
+                return ENOMEM;
+            ent->key_data = newptr;
 
 /* BEGIN CSTYLED */
             for (j = prev_n_keys; j < ent->n_key_data; j++) {
@@ -713,7 +711,6 @@ ulog_conv_2dbentry(krb5_context context, krb5_db_entry **entry,
                 }
 
                 for (cnt = 0; cnt < kp->key_data_ver; cnt++) {
-                    void *newptr;
                     kp->key_data_type[cnt] =  (krb5_int16)kv->k_enctype.k_enctype_val[cnt];
                     kp->key_data_length[cnt] = (krb5_int16)kv->k_contents.k_contents_val[cnt].utf8str_t_len;
                     newptr = realloc(kp->key_data_contents[cnt],
@@ -796,7 +793,8 @@ ulog_free_entries(kdb_incr_update_t *updates, int no_of_updates)
 {
 
     kdb_incr_update_t *upd;
-    int i, j, k, cnt;
+    unsigned int i, j;
+    int k, cnt;
 
     if (updates == NULL)
         return;
