@@ -12,6 +12,7 @@
 
 #include "k5-thread.h"
 #include "k5-platform.h"
+#include "k5-plugin.h"
 #include "com_err.h"
 #include "profile.h"
 
@@ -35,7 +36,6 @@ struct _prf_data_t {
 	unsigned long	frac_ts;   /* fractional part of timestamp, if any */
 	int		flags;	/* r/w, dirty */
 	int		upd_serial; /* incremented when data changes */
-	char		*comment;
 
 	size_t		fslen;
 
@@ -73,13 +73,21 @@ typedef struct _prf_file_t *prf_file_t;
 /*
  * The profile flags
  *
- * Deprecated use of read/write profile flag.  
+ * Deprecated use of read/write profile flag.
  * Check whether file is writable lazily so we don't call access as often.
  */
 #define PROFILE_FILE_DEPRECATED_RW	0x0001
 #define PROFILE_FILE_DIRTY		0x0002
 #define PROFILE_FILE_SHARED		0x0004
 
+struct _prf_lib_handle_t {
+	k5_mutex_t lock;
+	int refcount;
+	struct plugin_file_handle *plugin_handle;
+};
+
+typedef struct _prf_lib_handle_t *prf_lib_handle_t;
+
 /*
  * This structure defines the high-level, user visible profile_t
  * object, which is used as a handle by users who need to query some
@@ -88,6 +96,11 @@ typedef struct _prf_file_t *prf_file_t;
 struct _profile_t {
 	prf_magic_t	magic;
 	prf_file_t	first_file;
+
+	/* If non-null, use vtable operations instead of native ones. */
+	struct profile_vtable *vt;
+	void		*cbdata;
+	prf_lib_handle_t lib_handle;
 };
 
 /*
@@ -108,7 +121,7 @@ struct _profile_t {
 /* profile_parse.c */
 
 errcode_t profile_parse_file
-	(FILE *f, struct profile_node **root);
+	(FILE *f, struct profile_node **root, char **ret_modspec);
 
 errcode_t profile_write_tree_file
 	(struct profile_node *root, FILE *dstfile);
@@ -136,7 +149,7 @@ errcode_t profile_add_node
 
 errcode_t profile_make_node_final
 	(struct profile_node *node);
-	
+
 int profile_is_node_final
 	(struct profile_node *node);
 
@@ -161,11 +174,11 @@ errcode_t profile_find_node_subsection
 	(struct profile_node *section,
 		    const char *name, void **state,
 		    char **ret_name, struct profile_node **subsection);
-		   
+
 errcode_t profile_get_node_parent
 	(struct profile_node *section,
 		   struct profile_node **parent);
-		   
+
 errcode_t profile_delete_node_relation
 	(struct profile_node *section, const char *name);
 
@@ -198,14 +211,15 @@ errcode_t profile_rename_node
 errcode_t KRB5_CALLCONV profile_copy (profile_t, profile_t *);
 
 errcode_t profile_open_file
-	(const_profile_filespec_t file, prf_file_t *ret_prof);
+	(const_profile_filespec_t file, prf_file_t *ret_prof,
+	 char **ret_modspec);
 
-#define profile_update_file(P) profile_update_file_data((P)->data)
+#define profile_update_file(P, M) profile_update_file_data((P)->data, M)
 errcode_t profile_update_file_data
-	(prf_data_t profile);
-#define profile_update_file_locked(P) profile_update_file_data_locked((P)->data)
+	(prf_data_t profile, char **ret_modspec);
+#define profile_update_file_locked(P, M) profile_update_file_data_locked((P)->data, M)
 errcode_t profile_update_file_data_locked
-	(prf_data_t data);
+	(prf_data_t data, char **ret_modspec);
 
 #define profile_flush_file(P) (((P) && (P)->magic == PROF_MAGIC_FILE) ? profile_flush_file_data((P)->data) : PROF_MAGIC_FILE)
 errcode_t profile_flush_file_data
@@ -230,8 +244,8 @@ int profile_file_is_writable
 void profile_dereference_data (prf_data_t);
 void profile_dereference_data_locked (prf_data_t);
 
-int profile_lock_global (void);
-int profile_unlock_global (void);
+void profile_lock_global (void);
+void profile_unlock_global (void);
 
 /* prof_init.c -- included from profile.h */
 errcode_t profile_ser_size
@@ -246,8 +260,7 @@ errcode_t profile_ser_internalize
 /* prof_get.c */
 
 errcode_t profile_get_value
-	(profile_t profile, const char **names,
-		    const char	**ret_value);
+	(profile_t profile, const char **names, char **ret_value);
 /* Others included from profile.h */
-	
+
 /* prof_set.c -- included from profile.h */
